-----------------------
-- PARAMETER
-----------------------

SPACE_SIZE = 31 -- Try 31, 101

-----------------------
-- GLOBAL
-----------------------


-----------------------
-- MODEL
-----------------------


material = CellularSpace{
	xdim = SPACE_SIZE
}

material:createNeighborhood{
	strategy = "moore", -- Try vornneumann
	self = false
	
}

forEachCell(material, function(cell)
	local middle = math.floor( SPACE_SIZE/2 ) -- Try 2 column of vacancies in the middle instead of one
	
	a = cell:getX()
	
	if a < middle then
		cell.cover = "atom1"
	elseif a > middle then
		cell.cover = "atom2"
	else -- a == middle 
		cell.cover = "vacancy"
	end	
	
end)
material:synchronize()	

function calcConcentration(myCell) 
	local myCover = myCell.past.cover
	local counter 		= 0
	local counterLikeMe = 0
	forEachNeighbor( myCell, function( myCell, n)
		if	( n.past.cover == myCover ) then
				counterLikeMe = counterLikeMe + 1
		end
		counter = counter + 1
	end) 
	if( counter == 0 ) then
		return 0
	else 
		return counterLikeMe/counter
	end
end

update = function(cs)
	
	-- verification
	local vacancyCounter 	= 0
	local atom1Counter 		= 0
	local atom2Counter 		= 0
	local total = 0
	forEachCell(cs, function(cell)
	
	
		--# verification
		if( cell.past.cover == "atom1" ) then
			atom1Counter = atom1Counter + 1
		elseif ( cell.past.cover == "atom2" ) then
			atom2Counter = atom2Counter + 1
		else -- cell.past.cover == "vacancy"
			vacancyCounter = vacancyCounter + 1
		end 
		
		--# density rule 
		if cell.past.cover == "vacancy" then
			
			-- choose a random neighbor
			local neighbor = cell:getNeighborhood():sample()
			
			-- not moving empty neighbors
			if ( neighbor.past.cover ~= "vacancy" ) then 
			
				-- not moving twice the same atom
				if(  neighbor.cover ~= "vacancy" )then 
		
					--# move
					cell.cover = neighbor.past.cover
					neighbor.cover = "vacancy"		
				end						
			end
			
			--# calculate neighborhood concentration
			
			total = total + calcConcentration(cell)																
		end		
	end)
	
	--# calculate concentration in the grid
	totalConcentration = total / material:size()
	
	--delay_s(3)
	--print("vacancy:", vacancyCounter, "atom1:", atom1Counter, "atom2:" ,atom2Counter, "total:", total, "totalConcentration:", totalConcentration)
	
end --function update


t = Timer{
	Event{action = function()
		material:synchronize()
		update(material)
		material:notify()
	end}
}

-----------------------
-- SIMULATION
-----------------------

legend = Legend {
	grouping = "uniquevalue",
	type = "string",
	colorBar = {
		{value = "vacancy", color = "black"},
		{value = "atom1", color = "green"},
		{value = "atom2", color = "blue"},
	}
}

Observer{
	subject = material,
	attributes = {"cover"},
	legends = {legend}
}


material:notify()

--io.read()
t:execute(1000)
